/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



#ifndef AQDB_DB_H
#define AQDB_DB_H


#include <aqdatabase/aqdb.h>

#include <gwenhywfar/inherit.h>
#include <gwenhywfar/list1.h>
#include <gwenhywfar/list2.h>
#include <gwenhywfar/idlist64.h>
#include <gwenhywfar/url.h>


#ifdef __cplusplus
extern "C" {
#endif


typedef struct AQDB_DB AQDB_DB;
GWEN_INHERIT_FUNCTION_LIB_DEFS(AQDB_DB, AQDB_API)
GWEN_LIST_FUNCTION_LIB_DEFS(AQDB_DB, AQDB_DB, AQDB_API)
GWEN_LIST2_FUNCTION_LIB_DEFS(AQDB_DB, AQDB_DB, AQDB_API)


#include <aqdatabase/aqdb_object.h>
#include <aqdatabase/aqdb_column.h>
#include <aqdatabase/aqdb_sessiondescr.h>



AQDB_API
int AQDB_DbFactory(const char *url, AQDB_DB **pDb);



AQDB_API
void AQDB_DB_Attach(AQDB_DB *db);

AQDB_API
void AQDB_DB_free(AQDB_DB *db);


AQDB_API
const GWEN_URL *AQDB_DB_GetUrl(const AQDB_DB *db);


AQDB_API
int AQDB_DB_Open(AQDB_DB *db, uint32_t flags);

AQDB_API
int AQDB_DB_Create(AQDB_DB *db, uint32_t flags);

AQDB_API
int AQDB_DB_Close(AQDB_DB *db, uint32_t flags);


AQDB_API
int AQDB_DB_OpenTable(AQDB_DB *db,
		      const char *tname,
		      uint32_t flags,
		      AQDB_ID *pTableId);

AQDB_API
AQDB_COLUMN_LIST* AQDB_DB_GetColumnList(AQDB_DB *db, AQDB_ID tableId);


AQDB_API
int AQDB_DB_CreateTable(AQDB_DB *db,
			const char *tname,
			uint32_t flags,
			const AQDB_COLUMN_LIST *columns);

AQDB_API
int AQDB_DB_CloseTable(AQDB_DB *db,
                       AQDB_ID tableId,
		       uint32_t flags);

AQDB_API
int AQDB_DB_ReadObject(AQDB_DB *db,
		       AQDB_ID tableId,
		       AQDB_ID objectId,
		       AQDB_OBJECT **pObject);

AQDB_API
int AQDB_DB_WriteObject(AQDB_DB *db,
			const AQDB_OBJECT *object);

AQDB_API
int AQDB_DB_DeleteObject(AQDB_DB *db,
			 AQDB_ID tableId,
			 AQDB_ID id);

AQDB_API
int AQDB_DB_AddObject(AQDB_DB *db,
		      AQDB_ID tableId,
		      AQDB_OBJECT *object);

AQDB_API
int AQDB_DB_QuerySubmit(AQDB_DB *db,
			AQDB_ID tableId,
			AQDB_ID refQueryId,
			const char *queryExpression,
			uint32_t flags,
			AQDB_ID *pQueryId);

AQDB_API
int AQDB_DB_QueryGetFirst(AQDB_DB *db,
			  AQDB_ID queryId,
			  AQDB_ID *pIdListPtr,
			  int *pIdListLen);

AQDB_API
int AQDB_DB_QueryGetNext(AQDB_DB *db,
			 AQDB_ID queryId,
			 AQDB_ID *pIdListPtr,
			 int *pIdListLen);

AQDB_API
int AQDB_DB_QueryClose(AQDB_DB *db, AQDB_ID queryId);


AQDB_API
int AQDB_DB_BeginEdit(AQDB_DB *db, uint32_t flags);

AQDB_API
int AQDB_DB_EndEdit(AQDB_DB *db, uint32_t flags);


AQDB_API
int AQDB_DB_GetRollbackSessions(AQDB_DB *db, AQDB_SESSION_DESCR_LIST *sl);

AQDB_API
int AQDB_DB_UndoSession(AQDB_DB *db, AQDB_ID sid);


AQDB_API
int AQDB_DB_SetCache(AQDB_DB *db, uint32_t maxEntries, uint32_t maxSize);


#ifdef __cplusplus
}
#endif


#endif

