/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



#ifndef AQDB_DB_BE_H
#define AQDB_DB_BE_H

#include <aqdatabase/aqdb_db.h>
#include <aqdatabase/aqdb_column.h>



typedef int GWENHYWFAR_CB (*AQDB_DB_OPEN_FN)(AQDB_DB *db, uint32_t flags);
typedef int GWENHYWFAR_CB (*AQDB_DB_CREATE_FN)(AQDB_DB *db, uint32_t flags);
typedef int GWENHYWFAR_CB (*AQDB_DB_CLOSE_FN)(AQDB_DB *db, uint32_t flags);

typedef int GWENHYWFAR_CB (*AQDB_DB_OPENTABLE_FN)(AQDB_DB *db,
						  const char *tname,
						  uint32_t flags,
						  AQDB_ID *pTableId);
typedef int GWENHYWFAR_CB (*AQDB_DB_CREATETABLE_FN)(AQDB_DB *db,
						    const char *tname,
						    uint32_t flags,
						    const AQDB_COLUMN_LIST *columns);
typedef int GWENHYWFAR_CB (*AQDB_DB_CLOSETABLE_FN)(AQDB_DB *db,
						   AQDB_ID tableId,
						   uint32_t flags);

typedef int GWENHYWFAR_CB (*AQDB_DB_READOBJECT_FN)(AQDB_DB *db,
						   AQDB_ID tableId,
						   AQDB_ID objectId,
						   AQDB_OBJECT **pObject);
typedef int GWENHYWFAR_CB (*AQDB_DB_WRITEOBJECT_FN)(AQDB_DB *db,
						    const AQDB_OBJECT *object);

typedef int GWENHYWFAR_CB (*AQDB_DB_DELETEOBJECT_FN)(AQDB_DB *db,
						     AQDB_ID tableId,
						     AQDB_ID objectId);
typedef int GWENHYWFAR_CB (*AQDB_DB_ADDOBJECT_FN)(AQDB_DB *db,
						  AQDB_ID tableId,
						  AQDB_OBJECT *object);

typedef int GWENHYWFAR_CB (*AQDB_DB_BEGINEDIT_FN)(AQDB_DB *db, uint32_t flags);
typedef int GWENHYWFAR_CB (*AQDB_DB_ENDEDIT_FN)(AQDB_DB *db, uint32_t flags);

typedef int GWENHYWFAR_CB (*AQDB_DB_QUERYSUBMIT_FN)(AQDB_DB *db,
						    AQDB_ID tableId,
						    AQDB_ID refQueryId,
						    const char *queryExpression,
						    uint32_t flags,
						    AQDB_ID *pQueryId);

typedef int GWENHYWFAR_CB (*AQDB_DB_QUERYGETFIRST_FN)(AQDB_DB *db,
						      AQDB_ID queryId,
						      AQDB_ID *pIdListPtr,
						      int *pIdListLen);

typedef int GWENHYWFAR_CB (*AQDB_DB_QUERYGETNEXT_FN)(AQDB_DB *db,
						     AQDB_ID queryId,
						     AQDB_ID *pIdListPtr,
						     int *pIdListLen);

typedef int GWENHYWFAR_CB (*AQDB_DB_QUERYCLOSE_FN)(AQDB_DB *db, AQDB_ID queryId);


typedef int GWENHYWFAR_CB (*AQDB_DB_GETROLLBACKSESSIONS_FN)(AQDB_DB *db, AQDB_SESSION_DESCR_LIST *sl);
typedef int GWENHYWFAR_CB (*AQDB_DB_UNDOSESSION_FN)(AQDB_DB *db, AQDB_ID sid);


typedef AQDB_COLUMN_LIST* GWENHYWFAR_CB (*AQDB_DB_GETCOLUMNLIST_FN)(AQDB_DB *db, AQDB_ID tableId);


AQDB_API
AQDB_DB *AQDB_DB_new(const char *url);


AQDB_API
AQDB_DB_OPEN_FN
  AQDB_DB_SetOpenFn(AQDB_DB *db,
		    AQDB_DB_OPEN_FN f);

AQDB_API
AQDB_DB_CREATE_FN
  AQDB_DB_SetCreateFn(AQDB_DB *db,
		      AQDB_DB_CREATE_FN f);

AQDB_API
AQDB_DB_CLOSE_FN
  AQDB_DB_SetCloseFn(AQDB_DB *db,
		     AQDB_DB_CLOSE_FN f);

AQDB_API
AQDB_DB_OPENTABLE_FN
  AQDB_DB_SetOpenTableFn(AQDB_DB *db,
			 AQDB_DB_OPENTABLE_FN f);

AQDB_API
AQDB_DB_CREATETABLE_FN
  AQDB_DB_SetCreateTableFn(AQDB_DB *db,
			   AQDB_DB_CREATETABLE_FN f);

AQDB_API
AQDB_DB_CLOSETABLE_FN
  AQDB_DB_SetCloseTableFn(AQDB_DB *db,
			  AQDB_DB_CLOSETABLE_FN f);

AQDB_API
AQDB_DB_READOBJECT_FN
  AQDB_DB_SetReadObjectFn(AQDB_DB *db,
			  AQDB_DB_READOBJECT_FN f);

AQDB_API
AQDB_DB_WRITEOBJECT_FN
  AQDB_DB_SetWriteObjectFn(AQDB_DB *db,
			   AQDB_DB_WRITEOBJECT_FN f);

AQDB_API
AQDB_DB_DELETEOBJECT_FN
  AQDB_DB_SetDeleteObjectFn(AQDB_DB *db,
			    AQDB_DB_DELETEOBJECT_FN f);

AQDB_API
AQDB_DB_ADDOBJECT_FN
  AQDB_DB_SetAddObjectFn(AQDB_DB *db,
			 AQDB_DB_ADDOBJECT_FN f);

AQDB_API
AQDB_DB_QUERYSUBMIT_FN
  AQDB_DB_SetQuerySubmitFn(AQDB_DB *db,
                           AQDB_DB_QUERYSUBMIT_FN f);

AQDB_API
AQDB_DB_QUERYGETFIRST_FN
  AQDB_DB_SetQueryGetFirstFn(AQDB_DB *db,
			     AQDB_DB_QUERYGETFIRST_FN f);

AQDB_API
AQDB_DB_QUERYGETNEXT_FN
  AQDB_DB_SetQueryGetNextFn(AQDB_DB *db,
			    AQDB_DB_QUERYGETNEXT_FN f);

AQDB_API
AQDB_DB_QUERYCLOSE_FN
  AQDB_DB_SetQueryCloseFn(AQDB_DB *db,
			  AQDB_DB_QUERYCLOSE_FN f);

AQDB_API
AQDB_DB_BEGINEDIT_FN
  AQDB_DB_SetBeginEditFn(AQDB_DB *db,
			 AQDB_DB_BEGINEDIT_FN f);

AQDB_API
AQDB_DB_ENDEDIT_FN
  AQDB_DB_SetEndEditFn(AQDB_DB *db,
		       AQDB_DB_ENDEDIT_FN f);


AQDB_API
AQDB_DB_GETROLLBACKSESSIONS_FN
  AQDB_DB_SetGetRollbackSessionsFn(AQDB_DB *db,
                                   AQDB_DB_GETROLLBACKSESSIONS_FN f);

AQDB_API
AQDB_DB_UNDOSESSION_FN
  AQDB_DB_SetUndoSessionFn(AQDB_DB *db,
                           AQDB_DB_UNDOSESSION_FN f);

AQDB_API
AQDB_DB_GETCOLUMNLIST_FN
  AQDB_DB_SetGetColumnListFn(AQDB_DB *db,
			     AQDB_DB_GETCOLUMNLIST_FN f);





AQDB_API
int AQDB_DB_CacheGetObject(AQDB_DB *db,
                           AQDB_ID tableId,
                           AQDB_ID objectId,
                           AQDB_OBJECT **pObject);

AQDB_API
int AQDB_DB_CacheSetObject(AQDB_DB *db, const AQDB_OBJECT *object, uint32_t osize);

AQDB_API
void AQDB_DB_CachePurgeObject(AQDB_DB *db, AQDB_ID tableId, AQDB_ID objectId);




#endif






